# @push.rocks/smarturl
a url parsing lib

## Install

To install `@push.rocks/smarturl`, you'll need Node.js and npm on your system. Once you have those set up, run the following command in your terminal:

```bash
npm install @push.rocks/smarturl --save
```

This will add `@push.rocks/smarturl` to your project's dependencies.

## Usage

`@push.rocks/smarturl` is a library designed to simplify the parsing and manipulation of URLs within your TypeScript projects. Below is an overview of how you can leverage this library effectively in various scenarios.

### Basic URL Parsing

To start parsing URLs, you first need to import `@push.rocks/smarturl` into your TypeScript file:

```typescript
import { Smarturl } from '@push.rocks/smarturl';
```

Now, you can parse a URL string and access its components:

```typescript
const myUrl = Smarturl.createFromUrl('https://www.example.com:8080/path?query=string#hash');

console.log(myUrl.href); // https://www.example.com:8080/path?query=string#hash
console.log(myUrl.protocol); // https:
console.log(myUrl.host); // www.example.com:8080
console.log(myUrl.pathname); // /path
console.log(myUrl.search); // ?query=string
console.log(myUrl.hash); // #hash
console.log(myUrl.searchParams); // { query: 'string' }
```

### Modifying Search Parameters

`@push.rocks/smarturl` makes it easy to modify search parameters of a URL:

```typescript
// Creating a URL with initial search parameters
const myUrl = Smarturl.createFromUrl('https://www.example.com', {
  searchParams: {
    page: '1',
    filter: 'none',
  },
});

// Adding or updating search parameters
myUrl.searchParams['filter'] = 'newVal';
myUrl.searchParams['newParam'] = 'value';

console.log(myUrl.toString()); // https://www.example.com:443/?page=1&filter=newVal&newParam=value
```

### Complex URL Creation

Sometimes you may need to create a URL from parts:

```typescript
let myUrl = new Smarturl();
myUrl.protocol = 'https';
myUrl.hostname = 'www.example.com';
myUrl.port = '3000';
myUrl.pathname = '/path/to/resource';
myUrl.searchParams = {
  key: 'value',
  another: 'parameter'
};

console.log(myUrl.toString()); // Prints the full URL
```

### Handling Username and Password in URL

You can also include authentication details within the URL:

```typescript
const myUrl = Smarturl.createFromUrl('https://user:password@www.example.com');
console.log(myUrl.username); // user
console.log(myUrl.password); // password
```

### Working with URL Paths and Hashes

Manipulating paths and hashes is straightforward:

```typescript
const myUrl = Smarturl.createFromUrl('https://www.example.com/path/to/resource');
myUrl.hash = 'section1';

console.log(myUrl.toString()); // https://www.example.com:443/path/to/resource#section1
```

### URL Encoding and Decoding

When working with URLs, encoding and decoding of search parameters is handled automatically by the library. This ensures that the URL remains valid and interpretable by browsers and servers.

```typescript
const myUrl = Smarturl.createFromUrl('https://www.example.com');
myUrl.searchParams['redirect_uri'] = 'https://www.redirect.com/path';

console.log(myUrl.toString()); // Automatically encodes the URI
```

In addition to basic parsing and manipulation, `@push.rocks/smarturl` provides methods for deep manipulation of URLs, aiding in scenarios where complex URL operations are needed.

By understanding and utilizing these features, you can handle most URL-related tasks within your TypeScript applications with ease, ensuring that URLs are constructed, interpreted, and modified in a consistent and error-free manner.

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository. 

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
